import signal
import sys
import time

import typer

from .utils import get_proc, get_proc_data, HEADER, VERBOSE_HEADER


app = typer.Typer()


def _get_header(verbose: bool, color: bool) -> str:
    header = VERBOSE_HEADER if verbose else HEADER
    if color:
        header = typer.style(header.expandtabs(10), fg=typer.colors.GREEN, bold=True)
    else:
        header = header.expandtabs(10)
    return header


@app.command(help="Process monitor for linux")
def monitor(
    pid: int = typer.Argument(..., help="The pid of the process you want to monitor"),
    interval: float = typer.Option(0.5, min=0.1, help="The update interval in seconds"),
    verbose: bool = typer.Option(False, help="Use verbose output"),
    color: bool = typer.Option(True, help="Use colored output"),
):
    try:
        proc = get_proc(pid)
    except ValueError:
        typer.echo(f"The process does not exist: {pid}")
        raise typer.Exit(1)
    header = _get_header(verbose=verbose, color=color)
    typer.echo(header)
    i = 0
    while True:
        try:
            line = get_proc_data(proc, verbose=verbose)
        except ValueError:
            typer.echo(f"The process no longer exists: {pid}")
            raise typer.Exit(1)
        typer.echo(line.expandtabs(10))
        time.sleep(interval)
        i += 1
        if i == 20:
            typer.echo(header)
            i = 0


# handle Ctrl-C
def sigint_handler(sig, frame):
    sys.exit(0)


signal.signal(signal.SIGINT, sigint_handler)



from typing import Any
from typing import Dict
from typing import List
from typing import Tuple

import psutil


__all__: List[str] = [
    "get_proc",
    "get_proc_data",
    "HEADER",
    "VERBOSE_HEADER",
]


SYMBOLS = ('K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y')
ATTRS = ("memory_full_info", "cpu_percent")
HEADER = "\t".join("rss,vms,shared,pss,swap,uss,cpu (%),mem (%)".split(","))
VERBOSE_HEADER = "\t".join("rss,vms,shared,text,lib,data,dirty,pss,swap,uss,cpu (%),mem (%)".split(","))


def to_bytes(n: int) -> str:
    prefix = {}
    for i, symbol in enumerate(SYMBOLS):
        prefix[symbol] = 1 << (i + 1) * 10
    for symbol in reversed(SYMBOLS):
        if n >= prefix[symbol]:
            value = float(n) / prefix[symbol]
            return f"{value:.2f}{symbol}"
    return f"{n}B"


def get_line(info: Dict[str, Any]) -> str:
    memory_full_info = info["memory_full_info"]._asdict()
    line = "\t".join([
        to_bytes(memory_full_info["rss"]),
        to_bytes(memory_full_info["vms"]),
        to_bytes(memory_full_info["shared"]),
        to_bytes(memory_full_info["pss"]),
        to_bytes(memory_full_info["swap"]),
        to_bytes(memory_full_info["uss"]),
        f"{info['cpu_percent']:.2f}",
        f"{info['memory_percent']:.2f}",
    ])
    return line


def get_verbose_line(info: Dict[str, Any]) -> str:
    memory_full_info = info["memory_full_info"]._asdict()
    line = "\t".join([
        to_bytes(memory_full_info["rss"]),
        to_bytes(memory_full_info["vms"]),
        to_bytes(memory_full_info["shared"]),
        to_bytes(memory_full_info["text"]),
        to_bytes(memory_full_info["lib"]),
        to_bytes(memory_full_info["data"]),
        to_bytes(memory_full_info["dirty"]),
        to_bytes(memory_full_info["pss"]),
        to_bytes(memory_full_info["swap"]),
        to_bytes(memory_full_info["uss"]),
        f"{info['cpu_percent']:.2f}",
        f"{info['memory_percent']:.2f}",
    ])
    return line


def get_proc(pid: int) -> psutil.Process:
    try:
        proc = psutil.Process(pid)
    except psutil.NoSuchProcess:
        raise ValueError(f"The process does not exist: {pid}")
    else:
        return proc


def get_proc_data(proc: psutil.Process, verbose: bool = False, attrs: Tuple[str] = ATTRS) -> str:
    try:
        with proc.oneshot():
            info = proc.as_dict(attrs=attrs)
            info["memory_percent"] = proc.memory_percent("uss")
    except psutil.NoSuchProcess:
        raise ValueError(f"The process no longer exists: {proc.pid}")
    else:
        if verbose:
            line = get_verbose_line(info)
        else:
            line = get_line(info)
        return line



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pmon']

package_data = \
{'': ['*']}

install_requires = \
['psutil>=5.7.3,<6.0.0', 'typer[all]>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['pmon = pmon.cli:app']}

setup_kwargs = {
    'name': 'pmon',
    'version': '0.1.1',
    'description': 'A process monitor for linux',
    'long_description': '## pmon\n\nA process monitor for linux\n\n```\n$ pmon 23245\n\nrss       vms       pss       swap      uss       cpu (%)   mem (%)\n1.06G     2.52G     1.05G     0B        1.05G     0.00      6.83\n1.06G     2.52G     1.05G     0B        1.05G     0.00      6.83\n1.12G     2.58G     1.11G     0B        1.11G     64.20     7.22\n1.12G     2.58G     1.11G     0B        1.11G     0.00      7.22\n```\n\n### Install\n\n```\npipx install pmon\n```\n',
    'author': 'Panos Mavrogiorgos',
    'author_email': 'pmav99@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/pmav99/pmon',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
